from Script.Layer.Layer import Layer
from Resources.sprites.grass.Grass import Grass
class GrassLyaer(Layer):
    def __init__(self,cols,rows):
        Layer.__init__(self,cols,rows)
        self.element=Grass()
        self.CreateLayer()
    def CreateLayer(self):
        for i in range(self.cols):
            for j in range(self.rows):
                self.myLayer[i,j]=self.element.GetElement()


from Script.Layer.Layer import Layer
from Resources.sprites.castle.Castle01 import Castle
from Script.assist.Array import Array
class CastleLayer(Layer):
    def __init__(self,cols,rows,array,blank,collider):
        Layer.__init__(self,cols,rows)
        self.array=Array()
        self.array.Set(array)
        self.element=Castle()
        self.CreateLayer(blank,collider)
    def CreateLayer(self,blank,collider):
        for i in range(self.cols):
            for j in range(self.rows):
                if self.array[i,j]!=0:
                    collider[i,j]=2
                    blank.myLayer[i,j]=self.element.GetElement()





from Script.Layer.GrassLayer import GrassLyaer
from Script.Layer.WaterLayer import WaterLayer
from Script.Layer.PineLayer import PineLayer
from Script.Layer.StoneLayer import StoneLayer
from Script.Layer.StumpLayer import StumpLayer
from Script.Layer.CastleLayer import CastleLayer
from Script.Layer.BackGroundLayer import BackGroundLayer
from Script.Layer.PlayerLayer import PlayerLayer
from Script.Layer.BlankLayer import BlankLayer
from Script.Layer.StumpLayer import StumpLayer
from Script.Layer.CollisionLayer import CollisionLayer
from Script.Layer.SoundsLayer import SoundsLayer
from Script.Layer.EquipageLayer import EquipageLayer
from Script.assist.Array import Array
import threading
import pygame
import time
from pygame.locals import *

class MixedLayer:
    _instance_lock = threading.Lock()
    def __init__(self,cols,rows):
        self.cols=cols
        self.rows=rows
        self.myLayer=Array()
        self.collider=CollisionLayer(cols,rows)
        self.grassLayer=GrassLyaer(self.cols,self.rows)
        self.substance=BlankLayer(self.cols,self.rows)
        self.layer=[]
        self.equList=[]
        self.colliderArray=Array(cols,rows)
        self.layer.append(self.grassLayer)
        self.layer.append(self.substance)
        self.sound=SoundsLayer()
        self.sound.CreateLayer()
        self.sound.PlaySound()
        self.CreateBackGround()
    def CreateBackGround(self):
        self.backGroundLayer=BackGroundLayer() 
    def CreateWater(self,array):
        self.waterLayer=WaterLayer(self.cols,self.rows,array,self.grassLayer,self.colliderArray)  
    def CreatePine(self,array):
        self.pineLayer=PineLayer(self.cols,self.rows,array,self.substance,self.colliderArray) 
    def CreateStone(self,array):
        self.stoneLayer=StoneLayer(self.cols,self.rows,array,self.substance,self.colliderArray) 
    def CreateStump(self,array):
        self.stumpLayer=StumpLayer(self.cols,self.rows,array,self.substance,self.colliderArray) 
    def CreateCastle(self,array):
        self.castleLayer=CastleLayer(self.cols,self.rows,array,self.substance,self.colliderArray) 
    def CreatePlayer(self,array):
        self.playerLayer=PlayerLayer(self.cols,self.rows,array)
    def CreateEquipage(self,array):
        self.EquipageLayer=EquipageLayer(self.cols,self.rows,array,self,self.colliderArray)
    def UpdateCollision(self):
        self.collider.UpdateCollision(self.colliderArray)
    def GetPlayer(self):
        return self.playerLayer.element
    def GetCollision(self):
        return self.collider.myLayer
    def StartMixed(self,screen):   
        while True:
            screen.blit(self.backGroundLayer.GetBackGround(),(0,0))
            for x in self:
                if x!=None:
                    self.myLayer.Set(x)
                    for i in range(0,x.cols):  
                        for j in range(x.rows):
                            a=int(j*65+(300-(x.rows/2*65))-10)
                            b=int(i*65+(300-(x.cols/2*65))+30)
                            if x.myLayer[i,j]:  
                                screen.blit(x.myLayer[i,j], (a,b))
                else:
                    break
            for x in self.equList:
                screen.blit(x.GetAnim(),(x.x,x.y))
            screen.blit(self.playerLayer.GetPlayer(),self.playerLayer.GetPlayerPos())
            time.sleep(1/30)
            pygame.display.update()
    @staticmethod
    def GetInstance():
        return MixedLayer._instance
    def __iter__(self):
        self.index=0
        return self
    def __next__(self):
        if self.index < len(self.layer):
            self.index+=1
            return self.layer[self.index-1]
        else:
            return None
    def __new__(cls, *args, **kwargs):
        if not hasattr(MixedLayer, "_instance"):
            with MixedLayer._instance_lock:
                if not hasattr(MixedLayer, "_instance"):
                    MixedLayer._instance = object.__new__(cls)  
        return MixedLayer._instance


from Maps.LevelBase import LevelBase
from Script.assist.Array import Array
class Level02(LevelBase):
    def __init__(self,screen,cols,rows):
        LevelBase.__init__(self,screen,cols,rows)
    def LoadLevel(self):
        array=Array(4,5)
        array[0,0] = 1
        array[0,1] = 1
        array[0,2] = 1
        array[0,3] = 1
        array[0,4] = 1
        array[1,0] = 1
        array[2,0] = 1
        array[3,2] = 1
        array[2,2] = 1
        array[1,4] = 1
        array[2,4] = 1
        array[3,4] = 1
        self.CreatePine(array)

        array.Clear()
        array[3,3] = 1
        self.CreateCastle(array)

        array.Clear()
        array[3,0] = 1
        self.CreatePlayer(array)
        self.UpdateCollision()




from Maps.LevelBase import LevelBase
from Script.assist.Array import Array
class Level03(LevelBase):
    def __init__(self,screen,cols,rows):
        LevelBase.__init__(self,screen,cols,rows)
    def LoadLevel(self):
        array=Array(7,8)
        array[0,0] = 1
        array[0,1] = 1
        array[0,2] = 1
        array[0,3] = 1
        array[0,4] = 1
        array[0,5] = 1
        array[0,6] = 1
        array[0,7] = 1
        array[6,0] = 1
        array[6,1] = 1
        array[6,2] = 1
        array[6,3] = 1
        array[6,4] = 1
        array[6,5] = 1
        array[6,6] = 1
        array[6,7] = 1
        array[1,0] = 1
        array[2,0] = 1
        array[4,0] = 1
        array[5,0] = 1
        self.CreatePine(array)
        array.Clear()
        array[2,3] = 1
        array[3,3] = 1
        array[2,4] = 1
        array[3,4] = 1
        array[3,2] = 1
        self.CreateWater(array)
        array.Clear()
        array[2,2] = 1
        array[4,5] = 1
        array[2,5] = 1
        self.CreateStone(array)
        array.Clear()
        array[1,7]=1
        array[2,7]=1
        array[4,7]=1
        array[5,7]=1
        self.CreateStump(array)
        array.Clear()
        array[3,7] = 1
        self.CreateCastle(array)
        array.Clear()
        array[1,1] = 1
        self.CreateEquipage(array)
        array.Clear()
        array[6,5] = 2
        self.CreateEquipage(array)
        array.Clear()
        array[6,1] = 3
        self.CreateEquipage(array)
        array.Clear()
        array[3,0] = 1
        self.CreatePlayer(array)
        self.UpdateCollision()





__all__=['BackGroundLayer','BlankLayer','CastleLayer','CollisionLayer','GrassLayer','Layer','MixedLayer','PineLayer','PlayerLayer','SoundsLayer','StoneLayer','StumpLayer','WaterLayer','EquipageLayer']


from Script.mcgame.MCGame import MCGame
import _thread
import unittest
import time
import Maps.MapManager

mygame=MCGame()

def SetMapName(name):
    mygame.LoadMap(name)

class mcgameunittest(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        pass
        #mygame.LoadMap(Maps.MapManager.LoadLevel01())
    def test_forword_run(self):
        mygame.forword()

    def test_left_run(self):
        mygame.left()

    def test_right_run(self):
        mygame.right()
        
    def test_updateScreen_run(self):
        mygame.Update()

    def test_pickup_run(self):
        mygame.pickup()




from Script.Layer.Layer import Layer
from Resources.sounds.Sounds import Sounds
from Script.assist.Array import Array
import pygame
import _thread
import time
class SoundsLayer():
    def __init__(self):
        self.sound=Sounds()
    def CreateLayer(self):
        pygame.mixer_music.load(self.sound.GetEement())
    def PlaySound(self):
        _thread.start_new_thread(self.Play,())
    def Play(self):
        while True:
            pygame.mixer_music.play()
            time.sleep(171)
            pygame.mixer_music.stop()


from Maps.Level01 import Level01
from Maps.Level02 import Level02
from Maps.Level03 import Level03
from Script.mcgame.MCGame import MCGame

def LoadLevel01():
    return Level01(MCGame.GetInstance().screen,3,3)
def LoadLevel02():
    return Level02(MCGame.GetInstance().screen,4,5)
def LoadLevel03():
    return Level03(MCGame.GetInstance().screen,7,8)



from Script.assist.Array import Array
class Layer:
    def __init__(self,cols,rows):
        self.cols=cols
        self.rows=rows
        self.element=None
        self.array=Array(cols,rows)
        self.myLayer=self.array()
    def GetLayer(self):
        return self.myLayer
    def CreateLayer(self):
        pass


from Script.assist.Array import Array
from Script.Layer.Layer import Layer
class BlankLayer(Layer):
    def __init__(self,cols,rows):
        Layer.__init__(self,cols,rows)
        self.element=Array(cols,rows)




import random
import pygame
from pygame.locals import *
import sys
import os
from Resources.sprites.sprite import Sprite

class Water:
    def __init__(self):
        Sprite.__init__(self)
        n=0
        for x in range(3):
            self.element.append(pygame.image.load(Get('water_' + str(n) + '.png')))
            n+=1
    def GetElement(self):
        return self.element[random.randint(0,2)]

def Get(name):
    return os.path.dirname(os.path.realpath(__file__))+"\\"+name



from setuptools import setup

setup(
    name='mc_knight',
    version='0.0.5',
    author='魔扣少儿编程_Hugn',
    author_email='wang1183478375@outlook.com',
    url ='https://www.coding4fun.com.cn/',
    install_requires=['pygame>=1.9.4'
                      ],
    packages = ['Maps',
                'Resources/collision',
                'Resources',
                'Resources/sounds',
                'Resources/sprites',
                'Resources/sprites/background',
                'Resources/sprites/castle',
                'Resources/sprites/equipage',
                'Resources/sprites/grass',
                'Resources/sprites/pine',
                'Resources/sprites/Player',
                'Resources/sprites/stone',
                'Resources/sprites/stump',
                'Resources/sprites/water',
                'Script',
                'Script/assist',
                'Script/equipage',
                'Script/Layer',
                'Script/mcgame',
                'Script/Player',
                'Script/Render'],
    data_files=[('',['mc_knight.py']),
                ('Resources/sounds',['Resources/sounds/sound_0.mp3']),
                ('Resources/sprites/background',['Resources/sprites/background/bg_0.png',
                                                 'Resources/sprites/background/bg_1.png']),
                ('Resources/sprites/castle',['Resources/sprites/castle/castle_0.png']),
                ('Resources/sprites/equipage',['Resources/sprites/equipage/knight_full_15fps_1.png']),
                ('Resources/sprites/grass',['Resources/sprites/grass/grass_0.png',
                                            'Resources/sprites/grass/grass_1.png',
                                            'Resources/sprites/grass/grass_2.png',
                                            'Resources/sprites/grass/grass_3.png']),
                ('Resources/sprites/pine',['Resources/sprites/pine/pine_0.png',
                                           'Resources/sprites/pine/pine_1.png',
                                           'Resources/sprites/pine/pine_2.png',
                                           'Resources/sprites/pine/pine_3.png']),
                ('Resources/sprites/Player',['Resources/sprites/Player/knight_full_15fps_0.png']),
                ('Resources/sprites/stone',['Resources/sprites/stone/stone_0.png',
                                            'Resources/sprites/stone/stone_1.png']),
                ('Resources/sprites/stump',['Resources/sprites/stump/stump_0.png']),
                ('Resources/sprites/water',['Resources/sprites/water/water_0.png',
                                            'Resources/sprites/water/water_1.png',
                                            'Resources/sprites/water/water_2.png'])
                ],
    include_package_data = True, 
    zip_safe=False,
    )